/*********************************************************************
MLDemos: A User-Friendly visualization toolkit for machine learning
Copyright (C) 2010  Basilio Noris
Contact: mldemos@b4silio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License,
version 3 as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/
#include "PCAFaces.h"
#include "cameraGrabber.h"
#include "sampleManager.h"
#include "eigenFaces.h"
#include <QDebug>
using namespace std;

SampleManager trainSamples;
void on_mouse( int event, int x, int y, int flags, void* param );
u32 select_object = 0;
CvRect selection;
CvPoint origin;
bool bConstrainSelection = false;
IplImage *image = 0;
CameraGrabber *grabber;
EigenFaces eigen;

PCAFaces::PCAFaces()
: ThreadID(0)
{
}

PCAFaces::~PCAFaces()
{
	Stop();
}

void PCAFaces::Start()
{
	qDebug() << "Starting threadID: " << ThreadID;
	if(ThreadID) return; // we're already running
	ThreadID = startTimer(1000 / 30);
	grabber = new CameraGrabber();
}

void PCAFaces::Stop()
{
	qDebug() << "Stopping threadID: " << ThreadID;
	if(!ThreadID) return; // we're not running anymore
	killTimer(ThreadID);
	ThreadID = 0;
	grabber->Kill();
	delete grabber;
	cvDestroyWindow("Input");
	trainSamples.Hide();
}

IplImage *display = 0;
void PCAFaces::timerEvent(QTimerEvent *event)
{
	if(grabber)
	{
		// grab image
		grabber->GrabFrame(&image);

		if(!display) display = cvCloneImage(image);
		else cvCopy(image, display);

		cvRectangle(display, cvPoint(selection.x, selection.y), cvPoint(selection.x+selection.width, selection.y+selection.height), CV_RGB(0,0,0),3);
		cvRectangle(display, cvPoint(selection.x, selection.y), cvPoint(selection.x+selection.width, selection.y+selection.height), CV_RGB(255,255,255));

		cvNamedWindow("Input");
		cvShowImage("Input", display);
		cvSetMouseCallback("Input", on_mouse, 0);

		if(trainSamples.GetCount()) trainSamples.Show();
		int c = cvWaitKey(1);
		if(c==' ')
		{
			trainSamples.AddSample(image, selection);
		}
		if(c=='s') trainSamples.Save("newSamples.png");
		if(c=='x')
		{
			eigen.Learn(trainSamples.GetSamples(),trainSamples.GetLabels());
			std::vector<float*> projections = eigen.GetProjections(2,true);
			vector<fvec> samples;
			ivec labels;
			fvec sample;sample.resize(2);
			FOR(i,projections.size())
			{
				sample[0] = projections[i][0]*0.9f + 0.05f;
				sample[1] = projections[i][1]*0.9f + 0.05f;
				samples.push_back(sample);
				labels.push_back(trainSamples.GetLabel(i));
			}
			emit(SetData(samples, labels, vector<ipair>()));
			FOR(i,projections.size()) KILL(projections[i]);
			projections.clear();
		}
	}
}

/****************************************************/
/*     mouse listener (for filter configuration)    */
/****************************************************/
void on_mouse( int event, int x, int y, int flags, void* param )
{
	if( !image )
		return;

	if( image->origin )
		y = image->height - y;

	if( select_object )
	{
		selection.x = MIN(x,origin.x);
		selection.y = MIN(y,origin.y);
		selection.width = selection.x + CV_IABS(x - origin.x);
		selection.height = selection.y + CV_IABS(y - origin.y);

		selection.x = MAX( selection.x, 0 );
		selection.y = MAX( selection.y, 0 );
		selection.width = MIN( selection.width, image->width );
		selection.height = MIN( selection.height, image->height );
		selection.width -= selection.x;
		selection.height -= selection.y;
		if(bConstrainSelection)
		{
			selection.width = min(selection.width, selection.height);
			selection.height = selection.width;
		}
	}

	switch( event )
	{
	case CV_EVENT_LBUTTONDOWN:
		origin = cvPoint(x,y);
		selection = cvRect(x,y,0,0);
		select_object = 1;
		break;
	case CV_EVENT_LBUTTONUP:
		select_object = 0;
		break;
	}
}

void PCAFaces::FetchResults(std::vector<fvec> results)
{

}

Q_EXPORT_PLUGIN2(IO_PCAFaces, PCAFaces)
