/*********************************************************************
MLDemos: A User-Friendly visualization toolkit for machine learning
Copyright (C) 2010  Basilio Noris
Contact: mldemos@b4silio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public License,
version 3 as published by the Free Software Foundation.

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/
#ifndef _DRAW_ELLIPSE_H
#define _DRAW_ELLIPSE_H
#include <public.h>
#include <basicMath.h>
#include <QPainter>

static void DrawEllipse(float *mean, float *sigma, float rad, QPainter *painter, QSize size)
{
	if(mean[0] != mean[0] || mean[1] != mean[1]) return; // nan
	float a = sigma[0], b = sigma[1], c = sigma[2];
	float L[4];
	L[0] = a; L[1] = 0; L[2] = b; L[3] = sqrtf(c*a-b*b);
	if(L[3] != L[3]) L[3] = 0;
	FOR(i,4) L[i] /= sqrtf(a);

	const int segments = 64;
	float oldX = FLT_MAX, oldY = FLT_MAX;
	for (float theta=0; theta <= PIf*2.f; theta += (PIf*2.f)/segments)
	{
		float x = cosf(theta)*rad;
		float y = sinf(theta)*rad;
		float nx = L[0]*x;
		float ny = L[2]*x + L[3]*y;
		nx += mean[0];
		ny += mean[1];
		if(oldX != FLT_MAX)
		{
			painter->drawLine(
				QPointF(nx*size.width(),ny*size.height()),
				QPointF(oldX*size.width(),oldY*size.height())
				);
		}
		oldX = nx;
		oldY = ny;
	}
}

#endif // _DRAW_ELLIPSE_H
