// Original code by Antonio Gulli, January 2011
// http://codingplayground.blogspot.com/

#ifndef EIGEN_PCA_H
#define EIGEN_PCA_H

#include <iostream>
#include <Eigen/Core>
#include <Eigen/Eigen>
#include <vector>

// Pretty simple and straightforward implementation of Kernel - PCA

using namespace Eigen;

typedef std::pair<double, int> myPair;
typedef std::vector<myPair> PermutationIndices;	


class Kernel
{
// We assume that the original dataset is represented in m x n matrix (m dimension, n points)
// so that points are represented by columns

// in this toy example we materialize the kernel, which is not necessary

public:
	const MatrixXd & get() const { return _kernel; };

	virtual void Compute(MatrixXd &data)
	{
		_kernel = MatrixXd::Zero(data.cols(), data.cols());
		// an example of kernel
		for (int i=0; i<data.cols();i++)
			for (int j=i; j <data.cols(); j++)
			{
				// an example of kernel projection
				double value = (data.col(i).dot(data.col(j)));
				_kernel(i,j) = value * value;
				_kernel(j,i) = value * value;
			}
	};

protected:
	MatrixXd _kernel;
};

class LinearKernel : public Kernel
{
public:
	void Compute(MatrixXd &data)
	{
		_kernel = MatrixXd::Zero(data.cols(), data.cols());
		for (int i=0; i<data.cols();i++)
			for (int j=i; j <data.cols(); j++)
			{
				// an example of kernel projection
				_kernel(i,j) = (data.col(i).dot(data.col(j)));
				_kernel(j,i) = _kernel(i,j);
			}
	}
};

class PolyKernel : public Kernel
{
	double degree;
public:
	PolyKernel(int degree) : degree(degree){};
	void Compute(MatrixXd &data)
	{
		_kernel = MatrixXd::Zero(data.cols(), data.cols());
		for (int i=0; i<data.cols();i++)
			for (int j=i; j <data.cols(); j++)
			{
				// an example of kernel projection
				double value = (data.col(i).dot(data.col(j)));
				_kernel(i,j) = pow(fabs(value), degree);
				_kernel(j,i) = _kernel(i,j);
			}
	}
};

class RBFKernel : public Kernel
{
	double gamma;
public:
	RBFKernel(float gamma) : gamma(gamma){};
	void Compute(MatrixXd &data)
	{
		_kernel = MatrixXd::Zero(data.cols(), data.cols());
		for (int i=0; i<data.cols();i++)
			for (int j=i; j <data.cols(); j++)
			{
				// an example of kernel projection
				double value = (data.col(i) - data.col(j)).transpose() * (data.col(i) - data.col(j));
				_kernel(i,j) = exp(-value*gamma);
				_kernel(j,i) = _kernel(i,j);
			}
	}
};

class PCA
{
public:
	int kernelType;
	int degree;
	float gamma;
	PCA() : kernelType(0), degree(2), gamma(0.01f){};
	//
	// compute the kernel pca
	//
	void kernel_pca(MatrixXd & dataPoints, unsigned int dimSpace);

	// get
	const MatrixXd & get() const { return _result; };

private:
	MatrixXd _result;
};


#endif // EIGEN_PCA_H
