// eigen_pca_kernel.cpp 

#include "eigen_pca.h"
#include <algorithm>

void PCA::kernel_pca(MatrixXd & dataPoints, unsigned int dimSpace)
{
	int m = dataPoints.rows();
	int n = dataPoints.cols();


	Kernel *k = 0;
	switch(kernelType)
	{
	case 0:
		k = new LinearKernel();
		break;
	case 1:
		k = new PolyKernel(degree);
		break;
	case 2:
		k = new RBFKernel(gamma);
		break;
	default:
		k = new Kernel();
	}
	k->Compute(dataPoints);

	//std::cout << "K:\n" << k->get() << std::endl;

	// ''centralize''
	MatrixXd K_Centralized = k->get()
							 - MatrixXd::Ones(n, n)*k->get()
							 - k->get()*MatrixXd::Ones(n, n)
							 + MatrixXd::Ones(n, n)*k->get()*MatrixXd::Ones(n, n);
	//std::cout << "Centralized" << std::endl;

	//std::cout << "K:\n" << k->get() << std::endl;

	// compute the eigenvalue on the K_Centralized Matrix
	EigenSolver<MatrixXd> m_solve(K_Centralized);
	//std::cout << "got the eigenvalues, eigenvectors" << std::endl;
	VectorXd eigenvalues = m_solve.eigenvalues().real();
	MatrixXd eigenVectors = m_solve.eigenvectors().real();	

	//std::cout << "eigv:\n" << eigenvalues << std::endl;
	//std::cout << "eigs:\n" << eigenVectors << std::endl;

	// sort and get the permutation indices
	PermutationIndices pi;
	for (int i = 0 ; i < n; i++)
		pi.push_back(std::make_pair(-eigenvalues(i), i));

	std::sort(pi.begin(), pi.end());

	// get top eigenvectors
	_result = MatrixXd::Zero(n, dimSpace);
	for (unsigned int i = 0; i < dimSpace; i++)
	{
		_result.col(i) = eigenVectors.col(pi[i].second); // permutation indices
		//_result.col(i) = eigenVectors.row(pi[i].second).transpose(); // permutation indices
	}

	MatrixXd sqrtE = MatrixXd::Zero(dimSpace, dimSpace);
	for (unsigned int i = 0; i < dimSpace; i++)
	{
		//sqrtE(i, i) = sqrt(-pi[i].first);
		sqrtE(i, i) = 0.9;
	}

	// get the final data projection
	_result = (sqrtE * _result.transpose()).transpose();
	delete k;
}


