/*********************************************************************
MLDemos: A User-Friendly visualization toolkit for machine learning
Copyright (C) 2010  Basilio Noris
Contact: mldemos@b4silio.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*********************************************************************/
#include "mldemos.h"
#include <QDebug>
#include <fstream>
#include <QPixmap>
#include <QBitmap>
#include <QSettings>
#include <QFileDialog>
#include "classifier.h"
#include "regressor.h"
#include <fstream>

using namespace std;

void MLDemos::SaveLayoutOptions()
{
	QCoreApplication::setOrganizationName("LASA");
	QCoreApplication::setApplicationName("MachineLearning");

	QSettings settings;
	settings.beginGroup("Gui");
	settings.setValue("geometry", saveGeometry());
	settings.setValue("windowState", saveState());
	settings.setValue("classGeometry", classifyDialog->saveGeometry());
	settings.setValue("regrGeometry", regressDialog->saveGeometry());
	settings.setValue("dynGeometry", dynamicDialog->saveGeometry());
	settings.setValue("clusterGeometry", clusterDialog->saveGeometry());
	settings.setValue("drawGeometry", drawToolbarWidget->saveGeometry());
	settings.setValue("displayGeometry", displayDialog->saveGeometry());
	settings.setValue("statsGeometry", statsDialog->saveGeometry());

	settings.setValue("ShowClassOptions", classifyDialog->isVisible());
	settings.setValue("ShowRegrOptions", regressDialog->isVisible());
	settings.setValue("ShowDynOptions", dynamicDialog->isVisible());
	settings.setValue("ShowClusterOptions", clusterDialog->isVisible());
	settings.setValue("ShowDrawOptions", drawToolbarWidget->isVisible());
	settings.setValue("ShowDisplayOptions", displayDialog->isVisible());
	settings.setValue("ShowStatsOptions", statsDialog->isVisible());
	settings.endGroup();

	settings.beginGroup("displayOptions");
	settings.setValue("infoCheck", displayOptions->infoCheck->isChecked());
	settings.setValue("mapCheck", displayOptions->mapCheck->isChecked());
	settings.setValue("modelCheck", displayOptions->modelCheck->isChecked());
	settings.setValue("samplesCheck", displayOptions->samplesCheck->isChecked());
	// settings.setValue("trajectoriesCheck", displayOptions->trajectoriesCheck->isChecked());
	// settings.setValue("singleClassCheck", displayOptions->singleclassCheck->isChecked());
	settings.endGroup();

	settings.beginGroup("drawingOptions");
	settings.setValue("infoCheck", drawToolbarContext1->randCombo->currentIndex());
	settings.setValue("spinCount", drawToolbarContext1->spinCount->value());
	settings.setValue("spinSize", drawToolbarContext1->spinSize->value());
	settings.setValue("spinAngle", drawToolbarContext2->spinAngle->value());
	settings.setValue("spinSigmaX", drawToolbarContext2->spinSigmaX->value());
	settings.setValue("spinSigmaY", drawToolbarContext2->spinSigmaY->value());
	settings.setValue("eraseCheck", drawToolbar->eraseButton->isChecked());
	settings.setValue("sprayCheck", drawToolbar->sprayButton->isChecked());
	settings.setValue("singleCheck", drawToolbar->singleButton->isChecked());
	settings.setValue("ellipseCheck", drawToolbar->ellipseButton->isChecked());
	settings.setValue("lineCheck", drawToolbar->lineButton->isChecked());
	settings.setValue("trajectoryCheck", drawToolbar->trajectoryButton->isChecked());
	settings.setValue("infoCheck", drawToolbarContext1->randCombo->currentIndex());
	settings.endGroup();

	settings.beginGroup("classificationOptions");
	settings.setValue("positiveClass", optionsClassify->positiveSpin->value());
	settings.setValue("foldCount", optionsClassify->foldCountSpin->value());
	settings.setValue("trainRatio", optionsClassify->traintestRatioCombo->currentIndex());
	settings.setValue("tab", optionsClassify->tabWidget->currentIndex());
	settings.endGroup();

	settings.beginGroup("regressionOptions");
	settings.setValue("foldCount", optionsRegress->foldCountSpin->value());
	settings.setValue("trainRatio", optionsRegress->traintestRatioCombo->currentIndex());
	settings.setValue("tab", optionsRegress->tabWidget->currentIndex());
	settings.endGroup();

	settings.beginGroup("dynamicalOptions");
	settings.setValue("centerType", optionsDynamic->centerCombo->currentIndex());
	settings.setValue("zeroCheck", optionsDynamic->zeroCheck->isChecked());
	settings.setValue("resampleType", optionsDynamic->resampleCombo->currentIndex());
	settings.setValue("resampleCount", optionsDynamic->resampleSpin->value());
	settings.setValue("dT", optionsDynamic->dtSpin->value());
	settings.setValue("tab", optionsDynamic->tabWidget->currentIndex());
	settings.endGroup();

	settings.beginGroup("clusterOptions");
	settings.setValue("tab", optionsCluster->tabWidget->currentIndex());
	settings.endGroup();

	settings.beginGroup("statsOptions");
	settings.setValue("tab", showStats->tabWidget->currentIndex());
	settings.endGroup();

	FOR(i,classifiers.size())
	{
		if(!classifiers[i]) continue;
		settings.beginGroup(QString("plugins::classifiers::") + classifiers[i]->GetName());
		classifiers[i]->SaveOptions(settings);
		settings.endGroup();
	}
	FOR(i,clusterers.size())
	{
		if(!clusterers[i]) continue;
		settings.beginGroup(QString("plugins::clusterers::") + clusterers[i]->GetName());
		clusterers[i]->SaveOptions(settings);
		settings.endGroup();
	}
	FOR(i,regressors.size())
	{
		if(!classifiers[i]) continue;
		settings.beginGroup(QString("plugins::regressors::") + regressors[i]->GetName());
		regressors[i]->SaveOptions(settings);
		settings.endGroup();
	}
	FOR(i,dynamicals.size())
	{
		if(!classifiers[i]) continue;
		settings.beginGroup(QString("plugins::dynamicals::") + dynamicals[i]->GetName());
		dynamicals[i]->SaveOptions(settings);
		settings.endGroup();
	}



}

void MLDemos::LoadLayoutOptions()
{
	QCoreApplication::setOrganizationName("LASA");
	QCoreApplication::setApplicationName("MachineLearning");

	QSettings settings;
	settings.beginGroup("Gui");
	if(settings.contains("geometry")) restoreGeometry(settings.value("geometry").toByteArray());
#ifdef MACX // ugly hack to avoid resizing problems on the mac
	if(height() < 600) resize(width(),600);
#endif // MACX
	if(settings.contains("windowState")) restoreState(settings.value("windowState").toByteArray());
	if(settings.contains("classGeometry")) classifyDialog->restoreGeometry(settings.value("classGeometry").toByteArray());
	if(settings.contains("regrGeometry")) regressDialog->restoreGeometry(settings.value("regrGeometry").toByteArray());
	if(settings.contains("dynGeometry")) dynamicDialog->restoreGeometry(settings.value("dynGeometry").toByteArray());
	if(settings.contains("clusterGeometry")) clusterDialog->restoreGeometry(settings.value("clusterGeometry").toByteArray());
	if(settings.contains("drawGeometry")) drawToolbarWidget->restoreGeometry(settings.value("drawGeometry").toByteArray());
	if(settings.contains("displayGeometry")) displayDialog->restoreGeometry(settings.value("displayGeometry").toByteArray());
	if(settings.contains("statsGeometry")) statsDialog->restoreGeometry(settings.value("statsGeometry").toByteArray());

	if(settings.contains("ShowClassOptions")) classifyDialog->setVisible(settings.value("ShowClassOptions").toBool());
	if(settings.contains("ShowRegrOptions")) regressDialog->setVisible(settings.value("ShowRegrOptions").toBool());
	if(settings.contains("ShowDynOptions")) dynamicDialog->setVisible(settings.value("ShowDynOptions").toBool());
	if(settings.contains("ShowClusterOptions")) clusterDialog->setVisible(settings.value("ShowClusterOptions").toBool());
	if(settings.contains("ShowDrawOptions")) drawToolbarWidget->setVisible(settings.value("ShowDrawOptions").toBool());
	if(settings.contains("ShowDisplayOptions")) displayDialog->setVisible(settings.value("ShowDisplayOptions").toBool());
	if(settings.contains("ShowStatsOptions")) statsDialog->setVisible(settings.value("ShowStatsOptions").toBool());
	settings.endGroup();

	actionClassifiers->setChecked(classifyDialog->isVisible());
	actionRegression->setChecked(regressDialog->isVisible());
	actionDynamical->setChecked(dynamicDialog->isVisible());
	actionClustering->setChecked(clusterDialog->isVisible());
	actionDrawSamples->setChecked(drawToolbarWidget->isVisible());
	actionDisplayOptions->setChecked(displayDialog->isVisible());

	settings.beginGroup("displayOptions");
	if(settings.contains("infoCheck")) displayOptions->infoCheck->setChecked(settings.value("infoCheck").toBool());
	if(settings.contains("mapCheck")) displayOptions->mapCheck->setChecked(settings.value("mapCheck").toBool());
	if(settings.contains("modelCheck")) displayOptions->modelCheck->setChecked(settings.value("modelCheck").toBool());
	if(settings.contains("samplesCheck")) displayOptions->samplesCheck->setChecked(settings.value("samplesCheck").toBool());
	// if(settings.contains("trajectoriesCheck")) displayOptions->trajectoriesCheck->setChecked(settings.value("trajectoriesCheck").toBool());
	// if(settings.contains("singleClassCheck")) displayOptions->singleclassCheck->setChecked(settings.value("singleClassCheck").toBool());
	settings.endGroup();

	settings.beginGroup("drawingOptions");
	if(settings.contains("infoCheck")) drawToolbarContext1->randCombo->setCurrentIndex(settings.value("infoCheck").toInt());
	if(settings.contains("spinAngle")) drawToolbarContext2->spinAngle->setValue(settings.value("spinAngle").toFloat());
	if(settings.contains("spinSize")) drawToolbarContext1->spinSize->setValue(settings.value("spinSize").toFloat());
	if(settings.contains("spinCount")) drawToolbarContext1->spinCount->setValue(settings.value("spinCount").toFloat());
	if(settings.contains("spinSigmaX")) drawToolbarContext2->spinSigmaX->setValue(settings.value("spinSigmaX").toFloat());
	if(settings.contains("spinSigmaY")) drawToolbarContext2->spinSigmaY->setValue(settings.value("spinSigmaY").toFloat());
	if(settings.contains("eraseCheck")) drawToolbar->eraseButton->setChecked(settings.value("eraseCheck").toBool());
	if(settings.contains("sprayCheck")) drawToolbar->sprayButton->setChecked(settings.value("sprayCheck").toBool());
	if(settings.contains("singleCheck")) drawToolbar->singleButton->setChecked(settings.value("singleCheck").toBool());
	if(settings.contains("ellipseCheck")) drawToolbar->ellipseButton->setChecked(settings.value("ellipseCheck").toBool());
	if(settings.contains("lineCheck")) drawToolbar->lineButton->setChecked(settings.value("lineCheck").toBool());
	if(settings.contains("trajectoryCheck")) drawToolbar->trajectoryButton->setChecked(settings.value("trajectoryCheck").toBool());
	settings.endGroup();

	settings.beginGroup("classificationOptions");
	if(settings.contains("positiveClass")) optionsClassify->positiveSpin->setValue(settings.value("positiveClass").toFloat());
	if(settings.contains("foldCount")) optionsClassify->foldCountSpin->setValue(settings.value("foldCount").toFloat());
	if(settings.contains("trainRatio")) optionsClassify->traintestRatioCombo->setCurrentIndex(settings.value("trainRatio").toInt());
	if(settings.contains("tab")) optionsClassify->tabWidget->setCurrentIndex(settings.value("tab").toInt());
	settings.endGroup();

	settings.beginGroup("regressionOptions");
	if(settings.contains("foldCount")) optionsRegress->foldCountSpin->setValue(settings.value("foldCount").toFloat());
	if(settings.contains("trainRatio")) optionsRegress->traintestRatioCombo->setCurrentIndex(settings.value("trainRatio").toInt());
	if(settings.contains("tab")) optionsRegress->tabWidget->setCurrentIndex(settings.value("tab").toInt());
	settings.endGroup();

	settings.beginGroup("dynamicalOptions");
	if(settings.contains("centerType")) optionsDynamic->centerCombo->setCurrentIndex(settings.value("centerType").toInt());
	if(settings.contains("zeroCheck")) optionsDynamic->zeroCheck->setChecked(settings.value("zeroCheck").toBool());
	if(settings.contains("resampleType")) optionsDynamic->resampleCombo->setCurrentIndex(settings.value("resampleType").toInt());
	if(settings.contains("resampleCount")) optionsDynamic->resampleSpin->setValue(settings.value("resampleCount").toFloat());
	if(settings.contains("dT")) optionsDynamic->dtSpin->setValue(settings.value("dT").toFloat());
	if(settings.contains("tab")) optionsDynamic->tabWidget->setCurrentIndex(settings.value("tab").toInt());
	settings.endGroup();

	settings.beginGroup("clusterOptions");
	if(settings.contains("tab")) optionsCluster->tabWidget->setCurrentIndex(settings.value("tab").toInt());
	settings.endGroup();

	settings.beginGroup("statsOptions");
	if(settings.contains("tab")) showStats->tabWidget->setCurrentIndex(settings.value("tab").toInt());
	settings.endGroup();

	FOR(i,classifiers.size())
	{
		if(!classifiers[i]) continue;
		settings.beginGroup(QString("plugins::classifiers::") + classifiers[i]->GetName());
		classifiers[i]->LoadOptions(settings);
		settings.endGroup();
	}
	FOR(i,clusterers.size())
	{
		if(!clusterers[i]) continue;
		settings.beginGroup(QString("plugins::clusterers::") + clusterers[i]->GetName());
		clusterers[i]->LoadOptions(settings);
		settings.endGroup();
	}
	FOR(i,regressors.size())
	{
		if(!regressors[i]) continue;
		settings.beginGroup(QString("plugins::regressors::") + regressors[i]->GetName());
		regressors[i]->LoadOptions(settings);
		settings.endGroup();
	}
	FOR(i,dynamicals.size())
	{
		if(!dynamicals[i]) continue;
		settings.beginGroup(QString("plugins::dynamicals::") + dynamicals[i]->GetName());
		dynamicals[i]->LoadOptions(settings);
		settings.endGroup();
	}
}


void MLDemos::SaveParams( QString filename )
{
	if(!classifier && !regressor && !clusterer) return;

	ofstream file(filename.toAscii(), ios::out | ios::app);
	if(!file.is_open()) return;

	char groupName[255];

	if(classifier)
	{
		int tab = optionsClassify->tabWidget->currentIndex();
		sprintf(groupName,"classificationOptions");
		file << groupName << ":" << "tab" << " " << optionsClassify->tabWidget->currentIndex() << std::endl;
		file << groupName << ":" << "positiveClass" << " " << optionsClassify->positiveSpin->value() << std::endl;
		if(tab < classifiers.size() && classifiers[tab])
		{
			classifiers[tab]->SaveParams(file);
		}
	}
	if(regressor)
	{
		int tab = optionsRegress->tabWidget->currentIndex();
		sprintf(groupName,"regressionOptions");
		file << groupName << ":" << "tab" << " " << optionsRegress->tabWidget->currentIndex() << std::endl;
		if(tab < regressors.size() && regressors[tab])
		{
			regressors[tab]->SaveParams(file);
		}
	}
	if(dynamical)
	{
		int tab = optionsDynamic->tabWidget->currentIndex();
		sprintf(groupName,"dynamicalOptions");
		file << groupName << ":" << "centerType" << " " << optionsDynamic->centerCombo->currentIndex() << std::endl;
		file << groupName << ":" << "zeroCheck" << " " << optionsDynamic->zeroCheck->isChecked() << std::endl;
		file << groupName << ":" << "resampleType" << " " << optionsDynamic->resampleCombo->currentIndex() << std::endl;
		file << groupName << ":" << "resampleCount" << " " << optionsDynamic->resampleSpin->value() << std::endl;
		file << groupName << ":" << "dT" << " " << optionsDynamic->dtSpin->value() << std::endl;
		file << groupName << ":" << "tab" << " " << optionsDynamic->tabWidget->currentIndex() << std::endl;
		if(tab < dynamicals.size() && dynamicals[tab])
		{
			dynamicals[tab]->SaveParams(file);
		}
	}
	if(clusterer)
	{
		int tab = optionsCluster->tabWidget->currentIndex();
		sprintf(groupName,"clusterOptions");
		file << groupName << ":" << "tab" << " " << optionsCluster->tabWidget->currentIndex() << std::endl;
		if(tab < clusterers.size() && clusterers[tab])
		{
			clusterers[tab]->SaveParams(file);
		}
	}

	file.close();
}

bool startsWith(char *a, char *b)
{
	bool yeah = true;
	for (int i=0; i<strlen(b); i++)
	{
		yeah &= a[i] == b[i];
	}
	return yeah;
}

void MLDemos::LoadParams( QString filename )
{
	ifstream file(filename.toAscii());
	if(!file.is_open()) return;

	int sampleCnt, size;
	file >> sampleCnt;
	file >> size;
	char line[255];
	float value;

	char classGroup[255];
	char regrGroup[255];
	char dynGroup[255];
	char clustGroup[255];
	sprintf(classGroup,"classificationOptions");
	sprintf(regrGroup,"regressionOptions");
	sprintf(dynGroup,"dynamicalOptions");
	sprintf(clustGroup,"clusteringOptions");

	classifyDialog->setVisible(false);
	regressDialog->setVisible(false);
	dynamicDialog->setVisible(false);
	clusterDialog->setVisible(false);

	// we skip the samples themselves
	qDebug() << "Skipping "<< sampleCnt <<" samples" << endl;
	FOR(i, sampleCnt) file.getline(line,255);
	bool bClass = false, bRegr = false, bDyn = false, bClust = false;
	qDebug() << "Loading parameter list" << endl;
	int tab = 0;
	while(!file.eof())
	{
		file >> line;
		file >> value;
		qDebug() << line << " " << value << endl;
		if(startsWith(line, classGroup))
		{
			bClass = true;
			classifyDialog->setVisible(true);
			if(endsWith(line,"tab")) optionsClassify->tabWidget->setCurrentIndex(tab = (int)value);
			if(endsWith(line,"positiveClass")) optionsClassify->positiveSpin->setValue((int)value);
			if(tab < classifiers.size() && classifiers[tab]) classifiers[tab]->LoadParams(line,value);
		}
		if(startsWith(line, regrGroup))
		{
			bRegr = true;
			regressDialog->setVisible(true);
			if(endsWith(line,"tab")) optionsRegress->tabWidget->setCurrentIndex((int)value);
			if(tab < regressors.size() && regressors[tab]) regressors[tab]->LoadParams(line,value);
		}
		if(startsWith(line, dynGroup))
		{
			bDyn = true;
			dynamicDialog->setVisible(true);
			if(endsWith(line,"centerType")) optionsDynamic->centerCombo->setCurrentIndex((int)value);
			if(endsWith(line,"zeroCheck")) optionsDynamic->zeroCheck->setChecked((int)value);
			if(endsWith(line,"resampleType")) optionsDynamic->resampleCombo->setCurrentIndex((int)value);
			if(endsWith(line,"resampleCount")) optionsDynamic->resampleSpin->setValue((int)value);
			if(endsWith(line,"dT")) optionsDynamic->dtSpin->setValue((int)value);
			if(endsWith(line,"tab")) optionsDynamic->tabWidget->setCurrentIndex((int)value);
			if(tab < dynamicals.size() && dynamicals[tab]) dynamicals[tab]->LoadParams(line,value);
		}
		if(startsWith(line, clustGroup))
		{
			bClust = true;
			clusterDialog->setVisible(true);
			if(endsWith(line,"tab")) optionsCluster->tabWidget->setCurrentIndex((int)value);
			if(tab < clusterers.size() && clusterers[tab]) clusterers[tab]->LoadParams(line,value);
		}
	}
	file.close();
	ResetPositiveClass();
	if(bClass) Classify();
	if(bRegr) Regression();
	if(bDyn) Dynamize();
	if(bClust) Cluster();
	actionClassifiers->setChecked(classifyDialog->isVisible());
	actionRegression->setChecked(regressDialog->isVisible());
	actionDynamical->setChecked(dynamicDialog->isVisible());
	actionClustering->setChecked(clusterDialog->isVisible());
}
