#include <public.h>
#include <basicMath.h>
#include "basicOpenCV.h"
#include "sampleManager.h"


using namespace std;

u32 SampleManager::IDCount = 0;

SampleManager::SampleManager(CvSize resolution)
: size(resolution)
{
	ID = IDCount++;
	display = NULL;
	perm = NULL;
	bShowing = false;
}

SampleManager::~SampleManager()
{
	IMKILL(display);
	Clear();
}

void SampleManager::CreateSampleImage(IplImage **image, bool bShowLabels, float ratio)
{
	int cnt = samples.size();

	int gridH = (int)(sqrtf(cnt/ratio) + 0.5f);
	int gridW = (cnt / gridH) + (cnt%gridH ? 1 : 0);

	CvSize imSize = cvSize(size.width*gridW, size.height*gridH);
	if (!(*image) || (*image)->width != imSize.width || (*image)->height != imSize.height)
	{
		if((*image)) cvReleaseImage(image);
		(*image) = cvCreateImage(imSize, 8, 3);
	}
	cvZero((*image));

	FOR(i, cnt)
	{
		if (!samples.at(i)) continue;
		CvRect rect = cvRect((i%gridW) * size.width, (i/gridW) * size.height, size.width, size.height);
		cvSetImageROI((*image), rect);
		cvCopy(samples.at(i), (*image));
		cvResetImageROI((*image));
		if(bShowLabels && flags[i] == TEST)
		{
			cvSetImageROI((*image), rect);
			IplImage *black = cvCreateImage(size, 8, 3);
			cvZero(black);
			cvAddWeighted((*image),0.5,black,1,0,(*image));
			IMKILL(black);
			cvResetImageROI((*image));
		}
		if(bShowLabels && labels[i])
		{
			cvRectangle((*image), cvPoint(rect.x, rect.y), cvPoint(rect.x+rect.width-1, rect.y+rect.height-1), CV::color[labels[i]%CV::colorCnt], 1, CV_AA); 
		}
	}
}

void sm_on_mouse( int event, int x, int y, int flags, void* param );
int params[5];
void SampleManager::Show()
{
	int cnt = samples.size();
	if(!cnt) return;

	CreateSampleImage(&display, true);

	params[0] = (int)((void *)&display);
	params[1] = (int)((void *)&samples);
	params[2] = (int)((void *)&labels);
	params[3] = (int)((void *)&flags);
	params[4] = (int)((void *)&size);
	char name[255];
	sprintf(name, "collected samples %d", ID);
	cvNamedWindow(name);
	cvShowImage(name, display);
	cvSetMouseCallback(name, sm_on_mouse, (void *) &params);
	bShowing = true;
}

void SampleManager::Hide()
{
	char name[255];
	sprintf(name, "collected samples %d", ID);
	cvDestroyWindow(name);
	bShowing = false;
}

void sm_on_mouse( int event, int x, int y, int flags, void* param )
{
	IplImage *image = (*(IplImage **)(((int *)param)[0]));
	std::vector<IplImage *> *samples = (std::vector<IplImage *> *)(((int *)param)[1]);
	std::vector<u32> *labels = (std::vector<u32> *)(((int *)param)[2]);
	std::vector<smFlags> *smflags = (std::vector<smFlags> *)(((int *)param)[3]);
	CvSize size = (*(CvSize *)(((int *)param)[4]));
    if( !image )
        return;

    if( image->origin )
        y = image->height - y;

	x = x >= (0xffff>>1) ? -(0xffff - x + 1) : x;
	y = y >= (0xffff>>1) ? -(0xffff - y + 1) : y;

	if (x < 0) x = 0;
	if (y < 0) y = 0;

	unsigned int gridX = (image->width / size.width);
	unsigned int gridY = (image->height / size.height);
	unsigned int i = (int)(x / (float)image->width * gridX);
	unsigned int j = (int)(y / (float)image->height * gridY);
	unsigned int index = j*gridX + i;

	if(event == CV_EVENT_LBUTTONDOWN)
    {
	}
	else if(event == CV_EVENT_LBUTTONUP)
	{
		if(flags & CV_EVENT_FLAG_CTRLKEY)
		{
			if(samples->size() == 1)
			{
				IMKILL((*samples)[0]);
				samples->clear();
				labels->clear();
				cvZero(image);
			}
			else if(index < samples->size())
			{
				IMKILL((*samples)[index]);
				while(index < samples->size()-1)
				{
					(*samples)[index] = (*samples)[index+1];
					(*labels)[index] = (*labels)[index+1];
					index++;
				}
				samples->pop_back();
				labels->pop_back();
			}
		}
		else if(flags & CV_EVENT_FLAG_ALTKEY)
		{
			if(index < smflags->size())
			{
				if((*smflags)[index] == UNUSED)
				{
					(*smflags)[index] = TEST;
				}
				else if ((*smflags)[index] == TEST)
				{
					(*smflags)[index] = UNUSED;
				}
			}
		}
		else
		{
			if(index < labels->size())
			{
				if (flags & CV_EVENT_FLAG_SHIFTKEY)
				{
					u32 newLabel = ((*labels)[index]+1) % 256;
					for (u32 i=index; i<labels->size(); i++) (*labels)[i] = newLabel;
				}
				else (*labels)[index] = ((*labels)[index]+1) % 256;
			}
		}
    }
	else if(event == CV_EVENT_RBUTTONUP)
	{
		if(index < labels->size())
		{
				if (flags & CV_EVENT_FLAG_SHIFTKEY)
				{
					u32 newLabel = (*labels)[index] ? (*labels)[index]-1 : 255;
					for (u32 i=index; i<labels->size(); i++) (*labels)[i] = newLabel;
				}
				else (*labels)[index] = (*labels)[index] ? (*labels)[index]-1 : 255;
		}
	}
}


void SampleManager::Clear()
{
	FOR(i, samples.size())
	{
		IMKILL(samples[i]);
	}
	samples.clear();
	flags.clear();
	labels.clear();
	KILL(perm);
	if(display) cvZero(display);
}

void SampleManager::AddSample(IplImage *image, unsigned int label)
{
	if (!image) return;

	IplImage *img = cvCreateImage(size, 8, 3);
	if(image->nChannels == 3) cvResize(image, img, CV_INTER_CUBIC);
	else
	{
		IplImage *tmp = cvCreateImage(cvGetSize(image), 8, 3);
		cvCvtColor(image, tmp, CV_GRAY2BGR);
		cvResize(tmp, img);
		IMKILL(tmp);
	}
	samples.push_back(img);
	flags.push_back(UNUSED);
	labels.push_back(label);
	KILL(perm);
	perm = randPerm(samples.size());
}

void SampleManager::AddSample(IplImage *image, CvRect selection, unsigned int label)
{
	if (!image) return;
	if (selection.x < 0 || selection.y < 0 || !selection.width || !selection.height) return;
	if (selection.x + selection.width > image->width || selection.y + selection.height > image->height) return;

	ROI(image, selection);
	IplImage *img = cvCreateImage(size, 8, 3);
	cvResize(image, img, CV_INTER_CUBIC);
	unROI(image);
	samples.push_back(img);
	flags.push_back(UNUSED);
	labels.push_back(label);
	KILL(perm);
	perm = randPerm(samples.size());
}

void SampleManager::AddSamples(SampleManager newSamples)
{
	FOR(i, newSamples.GetSamples().size())
	{
		samples.push_back(newSamples.GetSample(i));
		flags.push_back(newSamples.GetFlag(i));
		labels.push_back(newSamples.GetLabel(i));
	}
	KILL(perm);
	perm = randPerm(samples.size());
}

void SampleManager::RemoveSample(unsigned int index)
{
	if(index >= samples.size()) return;
	if(samples.size() == 1)
	{
		Clear();
		return;
	}
	IMKILL(samples[index]);
	for (unsigned int i = index; i < samples.size()-1; i++)
	{
		samples[i] = samples[i+1];
		labels[i] = labels[i+1];
	}
	samples.pop_back();
	labels.pop_back();
}

void SampleManager::Randomize(int seed)
{
	KILL(perm);
	if(samples.size()) perm = randPerm(samples.size(), seed);
}

void SampleManager::ResetFlags()
{
	FOR(i, samples.size()) flags[i] = UNUSED;
}


std::vector<IplImage *> SampleManager::GetSamples(u32 count, smFlags flag, smFlags replaceWith)
{
	std::vector<IplImage *> selected;
	if (!samples.size() || !perm) return selected;

	if (!count)
	{
		FOR(i, samples.size())
		{
			if ( flags[perm[i]] == flag)
			{
				selected.push_back(samples[perm[i]]);
				flags[perm[i]] = replaceWith;
			}
		}
		return selected;
	}

	for ( u32 i=0, cnt=0; i < samples.size() && cnt < count; i++ )
	{
		if ( flags[perm[i]] == flag )
		{
			selected.push_back(samples[perm[i]]);
			flags[perm[i]] = replaceWith;
			cnt++;
		}
	}

	return selected;
}

void SampleManager::Save(const char *filename)
{
	if(!samples.size()) return;
	IplImage *image = NULL;

	IplImage *labelImg = cvCreateImage(size, 8, 3);
	u32 passes = 1 + samples.size() / (size.width*size.height);
	u32 cnt = min(size.width*size.height*3, (int)samples.size());
	cvZero(labelImg);
	samples.push_back(labelImg);
	FOR(i, passes)
	{
		labelImg = cvCreateImage(size, 8, 3);
		cvZero(labelImg);
		FOR(j, cnt)
		{
			labelImg->imageData[j] = labels[i*(size.width*size.height) + j];
		}
		samples.push_back(labelImg);
	}

	CreateSampleImage(&image);

	FOR(i, passes+1)
	{
		IMKILL(samples[samples.size()-1]);
		samples.pop_back();
	}

	cvSaveImage(filename, image);
	IMKILL(image);
}

bool SampleManager::Load(const char *filename, CvSize resolution)
{
	IplImage *image = cvLoadImage(filename);
	if(!image || image->width < resolution.width || image->height < resolution.height) return false;

	Clear();
	size = resolution;

	int gridW = image->width / resolution.width;
	int gridH = image->height / resolution.height;
	int cnt = gridW*gridH;
	bool bDone = false;
	FOR(i, cnt)
	{
		IplImage *sample = cvCreateImage(size, 8, 3);
		ROI(image, cvRect((i%gridW) * size.width, (i/gridW) * size.height, size.width, size.height));
		cvCopy(image, sample);
		unROI(image);

		if(bDone)
		{
			if(labels.size() == samples.size()) // we added all the labels already
			{
				IMKILL(sample);
				break;
			}
			u32 labelCnt = min((int)samples.size(), size.width*size.height*3);
			FOR(j, labelCnt)
			{
				labels.push_back((u8)sample->imageData[j]);
			}
			IMKILL(sample);
			continue;
		}

		if (cvSumPixels(sample) == 0)
		{
			IMKILL(sample);
			bDone = true;
		}
		else
		{
			samples.push_back(sample);
			flags.push_back(UNUSED);
			//labels.push_back(0);
		}
	}
	KILL(perm);
	perm = randPerm(samples.size());
	return samples.size() > 0;
}

u32 SampleManager::GetClassCount(std::vector<u32> classes)
{
	u32 *counts = new u32[256];
	memset(counts, 0, 256*sizeof(u32));
	FOR(i, classes.size()) counts[classes[i]]++;
	u32 result = 0;
	for (u32 i=1; i<256; i++) result += counts[i] > 0 ? 1 : 0;
	return result;
}

std::vector<bool> SampleManager::GetFreeFlags()
{
	std::vector<bool> res;
	FOR(i, flags.size()) res.push_back(flags[i] == UNUSED);
	return res;
}
