#ifndef _SAMPLE_MANAGER_H_
#define _SAMPLE_MANAGER_H_

#include <vector>

enum SampleManagerFlags
{
	UNUSED = 0x0000,
	TRAIN  = 0x0001,
	VALID  = 0x0010,
	TEST   = 0x0100
};
typedef SampleManagerFlags smFlags;

class SampleManager : public Filter
{
protected:
	static u32 IDCount;

	u32 ID;

	CvSize size; // the samples size (resolution)

	std::vector<IplImage *> samples;

	std::vector<smFlags> flags;

	std::vector<unsigned int> labels;

	u32 *perm;

	IplImage *display;

	void CreateSampleImage(IplImage **image, bool bShowLabels = false, float ratio = 3/2.f);

	bool bShowing;

public:
	SampleManager(CvSize resolution = cvSize(48,48));

	~SampleManager();
	
	void Show();

	void Hide();

	bool IsShowing(){return bShowing;};

	void Clear();

	void Randomize(int seed=-1);

	void AddSample(IplImage *image, unsigned int label = 0);

	void AddSample(IplImage *image, CvRect selection, unsigned int label = 0);

	void AddSamples(SampleManager newSamples);

	void RemoveSample(unsigned int index);

	CvSize GetSize(){return size;};

	int GetCount(){return samples.size();};

	IplImage *GetSample(unsigned int index=0){return index<samples.size() ? samples[index] : NULL;};

	std::vector<IplImage *> GetSamples(){return samples;};

	std::vector<IplImage *> GetSamples(u32 count, smFlags flag=UNUSED, smFlags replaceWith=TRAIN);

	void ResetFlags();

	smFlags GetFlag(unsigned int index){return index < flags.size() ? flags[index] : UNUSED;};

	std::vector<smFlags > GetFlags(){return flags;};

	std::vector<bool> GetFreeFlags();

	u32 GetLabel(unsigned int index){return index < labels.size() ? labels[index] : 0;};

	std::vector<unsigned int> GetLabels(){return labels;};
	
	void SetLabel(unsigned int index, unsigned int label){if(index<labels.size())labels[index] = label;};

	void Save(const char *filename);

	bool Load(const char *filename, CvSize resolution = cvSize(48,48));

	virtual void Config(IplImage *image, CvRect selection, IplImage * =0){AddSample(image, selection);};

	static u32 GetClassCount(std::vector<u32> classes);

};

#endif // _SAMPLE_MANAGER_H_